from rdflib import URIRef, Literal, Namespace, Graph
from rdflib.namespace import RDF, RDFS, OWL
import uuid


class OwlClass(object):
    """
    This class creates generic RDF classes that require only an rdfs:label
    property to be specified. It is mostly used by inheritance to produce
    specialised RDF classes, such as Report or Entity.
    """
    def __init__(self,
                 label,
                 uri=None,
                 comment=None):
        self.g = None
        self.label = label
        if uri:
            self.uri = uri
        else:
            self.uri = 'http://placeholder.org#' + str(uuid.uuid4())
        self.comment = comment

    def set_uri(self, uri):
        self.uri = uri

    def set_comment(self, comment):
        self.comment = comment

    def make_graph(self):
        """
        Constructs an RDF graph for this class using its instance variables

        :return: an rdflib Graph object
        """
        self.g = Graph()

        XSD = Namespace('http://www.w3.org/2001/XMLSchema#')

        self.g.add((URIRef(self.uri),
                    RDF.type,
                    OWL.Class))

        self.g.add((URIRef(self.uri),
                    RDFS.label,
                    Literal(self.label, datatype=XSD.string)))

        if self.comment:
            self.g.add((URIRef(self.uri),
                        RDFS.comment,
                        Literal(self.comment, datatype=XSD.string)))

    def get_graph(self):
        """
        Generates the RDF graph of this class

        :return: This class's RDF graph
        """
        if not self.g:
            self.make_graph()

        return self.g

    def serialize_graph(self, format='turtle'):
        return self.get_graph().serialize(format=format)


from rdflib import URIRef, Namespace
from rdflib.namespace import RDF
from pyproms.prov_entity import ProvEntity


# TODO: allow Entity - Entity relationships
class PromsEntity(ProvEntity):
    """
    Creates a PROMS-O Entity instance
    """
    def __init__(self,
                 label,
                 uri=None,
                 comment=None,
                 wasAttributedTo=None,
                 creator=None,
                 created=None,
                 licence=None,
                 confidentialityStatus=None,
                 metadataUri=None,
                 downloadURL=None):

        ProvEntity.__init__(self,
                            label,
                            uri=uri,
                            comment=comment,
                            wasAttributedTo=wasAttributedTo,
                            creator=creator,
                            created=created,
                            licence=licence,
                            metadataUri=metadataUri,
                            downloadURL=downloadURL)

        # if the PROMS Entities had properties that PROV Entity didn't have, we would do something with them here

    # overloaded
    def make_graph(self):
        """
        Specialises ProvEntity.make_graph()

        :return: an rdflib Graph object
        """
        ProvEntity.make_graph(self)

        # add in a type of PROMS Entity --> this is the only PROV/PROMS Entity difference for now
        PROMS = Namespace('http://promsns.org/def/proms#')
        self.g.add((URIRef(self.uri),
                    RDF.type,
                    PROMS.Entity))


from rdflib import URIRef, Namespace, Literal
from rdflib.namespace import RDF
import datetime
from pyproms.owlclass import OwlClass
from pyproms.prov_entity import ProvEntity
from pyproms.prov_agent import ProvAgent


class ProvActivity(OwlClass):
    """
    Creates a PROV-O Activity instance
    """
    def __init__(self,
                 label,
                 startedAtTime,
                 endedAtTime,
                 uri=None,
                 wasAssociatedWith=None,
                 comment=None,
                 used_entities=None,
                 generated_entities=None,
                 wasInformedBy=None):

        OwlClass.__init__(self, label, uri, comment)

        self.set_startedAtTime(startedAtTime)
        self.set_endedAtTime(endedAtTime)

        if wasAssociatedWith:
            self.set_wasAssociatedWith(wasAssociatedWith)
        else:
            self.wasAssociatedWith = None

        if used_entities:
            self.set_used_entities(used_entities)
        else:
            self.used_entities = None

        if generated_entities:
            self.set_generated_entities(generated_entities)
        else:
            self.generated_entities = None

        if wasInformedBy:
            self.set_wasInformedBy(wasInformedBy)
        else:
            self.wasInformedBy = None

    def set_wasAssociatedWith(self, wasAssociatedWith):
        if type(wasAssociatedWith) is ProvAgent:
            self.wasAssociatedWith = wasAssociatedWith
        else:
            raise TypeError('wasAssociatedWith must be an Agent, not a %s' % type(wasAssociatedWith))

    def set_startedAtTime(self, startedAtTime):
        if type(startedAtTime) is datetime.datetime:
            self.startedAtTime = startedAtTime
        else:
            raise TypeError('startedAtTime must be a datetime.datetime, not a %s' % type(startedAtTime))

    def set_endedAtTime(self, endedAtTime):
        if type(endedAtTime) is datetime.datetime:
            self.endedAtTime = endedAtTime
        else:
            raise TypeError('endedAtTime must be a datetime.datetime, not a %s' % type(endedAtTime))

    def set_used_entities(self, used_entities):
        if all(isinstance(n, ProvEntity) for n in used_entities):
            self.used_entities = used_entities
        else:
            raise TypeError('used_entities must be a list of Entity objects')

    def set_generated_entities(self, generated_entities):
        if all(isinstance(n, ProvEntity) for n in generated_entities):
            self.generated_entities = generated_entities
        else:
            raise TypeError('used_entities must be a list of Entity objects')

    def set_wasInformedBy(self, wasInformedBy):
        if type(wasInformedBy) is ProvActivity:
            self.wasInformedBy = wasInformedBy
        else:
            raise TypeError('wasAssociatedWith must be an Activity, not a %s' % type(wasInformedBy))

    def make_graph(self):
        """
        Specialises RdfClass.make_graph()

        :return: an rdflib Graph object
        """
        OwlClass.make_graph(self)

        XSD = Namespace('http://www.w3.org/2001/XMLSchema#')
        PROV = Namespace('http://www.w3.org/ns/prov#')

        self.g.add((URIRef(self.uri),
                    RDF.type,
                    PROV.Activity))

        if self.wasAssociatedWith:
            self.g = self.g + self.wasAssociatedWith.get_graph()
            self.g.add((URIRef(self.uri),
                        PROV.wasAssociatedWith,
                        URIRef(self.wasAssociatedWith.uri)))

        if self.startedAtTime:
            self.g.add((URIRef(self.uri),
                        PROV.startedAtTime,
                        Literal(self.startedAtTime.strftime("%Y-%m-%dT%H:%M:%S"), datatype=XSD.dateTime)))

        if self.endedAtTime:
            self.g.add((URIRef(self.uri),
                        PROV.endedAtTime,
                        Literal(self.endedAtTime.strftime("%Y-%m-%dT%H:%M:%S"), datatype=XSD.dateTime)))

        if self.used_entities:
            for used_entity in self.used_entities:
                #add the Entity to the graph
                self.g = self.g + used_entity.get_graph()
                #associate the Entity with the Activity
                self.g.add((URIRef(self.uri),
                            PROV.used,
                            URIRef(used_entity.uri)))

        if self.generated_entities:
            for generated_entity in self.generated_entities:
                #add the Entity to the graph
                self.g = self.g + generated_entity.get_graph()
                #associate the Entity with the Activity
                self.g.add((URIRef(self.uri),
                            PROV.generated,
                            URIRef(generated_entity.uri)))

        if self.wasInformedBy:
            self.g = self.g + self.wasInformedBy.get_graph()
            self.g.add((URIRef(self.uri),
                        PROV.wasInformedBy,
                        URIRef(self.wasInformedBy.uri)))

    def get_graph(self):
        """
        Generates the RDF graph of this class

        :return: This class's RDF graph
        """
        if not self.g:
            self.make_graph()

        return self.g


from pyproms.prov_activity import ProvActivity
from pyproms.prov_agent import ProvAgent
from pyproms.prov_entity import ProvEntity
from pyproms.proms_reportingsystem import PromsReportingSystem
from pyproms.proms_report_basic import PromsBasicReport
from pyproms.proms_report_external import PromsExternalReport
from pyproms.proms_report_internal import PromsInternalReport


from rdflib import URIRef, Namespace
from rdflib.namespace import RDF
from pyproms.prov_agent import ProvAgent


# TODO: convert this to a subclass of type Agent, not directly from RdfClass
class PromsReportingSystem(ProvAgent):
    """
    Creates a PROMS-O ReportingAystem instance
    """
    def __init__(self,
                 label,
                 uri=None,
                 comment=None,
                 name=None,
                 actedOnBehalfOf=None):

        # these variables are not allowed for a ReportingSystem specialisation of prov:Agent
        givenName = None
        familyName = None
        mbox = None

        ProvAgent.__init__(self,
                           label,
                           uri=uri,
                           comment=comment,
                           actedOnBehalfOf=actedOnBehalfOf,
                           name=name,
                           givenName=givenName,
                           familyName=familyName,
                           mbox=mbox)

    def make_graph(self):
        """
        Specialises ProvAgent.make_graph()

        :return: an rdflib Graph object
        """
        ProvAgent.make_graph(self)

        PROV = Namespace('http://www.w3.org/ns/prov#')
        PROMS = Namespace('http://promsns.org/def/proms#')

        self.g.add((URIRef(self.uri),
                    RDF.type,
                    PROV.Agent))

        self.g.add((URIRef(self.uri),
                    RDF.type,
                    PROMS.ReportingSystem))


from rdflib import URIRef, Namespace, Literal
from rdflib.namespace import RDF
from pyproms.proms_entity import PromsEntity
import datetime


# TODO: allow Entity - Entity relationships
class PromsServiceEntity(PromsEntity):
    """
    Creates a PROMS-O ServiceEntity instance
    """
    def __init__(self,
                 label,
                 serviceBaseUri,
                 query,
                 queriedAtTime,
                 cachedCopyUri=None,
                 uri=None,
                 comment=None,
                 wasAttributedTo=None,
                 creator=None,
                 created=None,
                 licence=None,
                 confidentialityStatus=None,
                 metadataUri=None,
                 downloadURL=None):

        PromsEntity.__init__(self,
                             label,
                             uri=uri,
                             comment=comment,
                             wasAttributedTo=wasAttributedTo,
                             creator=creator,
                             created=created,
                             licence=licence,
                             confidentialityStatus=confidentialityStatus,
                             metadataUri=metadataUri,
                             downloadURL=downloadURL)

        # the additional properties of a proms:ServiceEntity over a proms:Entity
        self.serviceBaseUri = serviceBaseUri
        self.query = query
        self.queriedAtTime = queriedAtTime
        self.cachedCopyUri = cachedCopyUri

    def set_serviceBaseUri(self, serviceBaseUri):
        self.serviceBaseUri = serviceBaseUri

    def set_query(self, query):
        self.query = query

    def set_queriedAtTime(self, queriedAtTime):
        if type(queriedAtTime) is datetime.datetime:
            self.queriedAtTime = queriedAtTime
        else:
            raise TypeError('startedAtTime must be a datetime.datetime, not a %s' % type(queriedAtTime))

    # TODO: add a type check constraint here and in constructor: URIRef or anyUri?
    def set_cachedCopyUri(self, cachedCopyUri):
        self.cachedCopyUri = cachedCopyUri

    # overloaded
    def make_graph(self):
        """
        Specialises PromsEntity.make_graph()

        :return: an rdflib Graph object
        """
        PromsEntity.make_graph(self)

        # add in a type of PROMS Entity --> this is the only PROV/PROMS Entity difference for now
        PROMS = Namespace('http://promsns.org/def/proms#')
        XSD = Namespace('http://www.w3.org/2001/XMLSchema#')
        self.g.add((URIRef(self.uri),
                    RDF.type,
                    PROMS.ServiceEntity))

        self.g.add((URIRef(self.uri),
                    PROMS.serviceBaseUri,
                    Literal(self.serviceBaseUri, datatype=XSD.anyUri)))

        self.g.add((URIRef(self.uri),
                    PROMS.query,
                    Literal(self.query, datatype=XSD.string)))

        self.g.add((URIRef(self.uri),
                    PROMS.queriedAtTime,
                    Literal(self.queriedAtTime.strftime("%Y-%m-%dT%H:%M:%S"), datatype=XSD.dateTime)))


from rdflib import URIRef, Literal, Namespace
from rdflib.namespace import RDF, FOAF
from pyproms.owlclass import OwlClass


# TODO: split Agent into People and Software Agents for more in-depth handling later
class ProvAgent(OwlClass):
    """
    Creates a PROV-O Agent instance
    """
    def __init__(self,
                 label,
                 uri=None,
                 comment=None,
                 actedOnBehalfOf=None,
                 name=None,
                 givenName=None,
                 familyName=None,
                 mbox=None):

        OwlClass.__init__(self, label, uri, comment)

        if actedOnBehalfOf:
            self.set_actedOnBehalfOf(actedOnBehalfOf)
        else:
            self.actedOnBehalfOf = None
        self.name = name
        self.givenName = givenName
        self.familyName = familyName
        self.mbox = mbox

    def set_actedOnBehalfOf(self, actedOnBehalfOf):
        if type(actedOnBehalfOf) is ProvAgent:
            self.actedOnBehalfOf = actedOnBehalfOf
        else:
            raise TypeError('wasAttributedTo must be an Agent, not a %s' % type(actedOnBehalfOf))

    def set_name(self, name):
        self.name = name

    def set_givenName(self, givenName):
        self.givenName = givenName

    def set_familyName(self, familyName):
        self.familyName = familyName

    def set_mbox(self, mbox):
        self.mbox = mbox

    def make_graph(self):
        """
        Specialises RdfClass.make_graph()

        :return: an rdflib Graph object
        """
        OwlClass.make_graph(self)

        PROV = Namespace('http://www.w3.org/ns/prov#')
        XSD = Namespace('http://www.w3.org/2001/XMLSchema#')

        self.g.add((URIRef(self.uri),
                    RDF.type,
                    PROV.Agent))

        if self.actedOnBehalfOf:
            self.g = self.g + self.actedOnBehalfOf.get_graph()
            self.g.add((URIRef(self.uri),
                        PROV.actedOnBehalfOf,
                        URIRef(self.actedOnBehalfOf.uri)))

        if self.name:
            self.g.add((URIRef(self.uri),
                        FOAF.name,
                        Literal(self.name, datatype=XSD.string)))

        if self.givenName:
            self.g.add((URIRef(self.uri),
                        FOAF.givenName,
                        Literal(self.givenName, datatype=XSD.string)))

        if self.familyName:
            self.g.add((URIRef(self.uri),
                        FOAF.familyName,
                        Literal(self.familyName, datatype=XSD.string)))

        if self.mbox:
            self.g.add((URIRef(self.uri),
                        FOAF.mbox,
                        Literal(self.mbox, datatype=XSD.string)))

    def get_graph(self):
        """
        Generates the RDF graph of this class

        :return: This class's RDF graph
        """
        if not self.g:
            self.make_graph()

        return self.g


from rdflib import URIRef, Namespace
from rdflib.namespace import RDF
from pyproms.owlclass import OwlClass
from pyproms.prov_activity import ProvActivity
import re


# TODO: fix the recursion issue wrt wasAssociatedWith values
class PromsActivity(ProvActivity):
    """
    Creates a PROMS-O Activity instance
    """
    def __init__(self,
                 label,
                 startedAtTime,
                 endedAtTime,
                 uri=None,
                 wasAssociatedWith=None,
                 comment=None,
                 used_entities=None,
                 generated_entities=None,
                 wasInformedBy=None,
                 namedActivityUri=None):

        PromsActivity.__init__(self,
                               label,
                               uri=uri,
                               wasAssociatedWith=wasAssociatedWith,
                               startedAtTime=startedAtTime,
                               endedAtTime=endedAtTime,
                               comment=comment,
                               used_entities=used_entities,
                               generated_entities=generated_entities,
                               wasInformedBy=wasInformedBy)

        # set PROMS Activity-only properties
        self.namedActivityUri = self.set_namedActivityUri(namedActivityUri)

    def set_namedActivityUri(self, namedActivityUri):
        """
        Sets a wasAssociatedWith property for this Activity to another Activity
        indicated via a URI

        :param namedActivityUri: URIRef or string
        :return: -
        """
        if type(namedActivityUri) is URIRef:
            self.namedActivityUri = str(namedActivityUri)
        elif type(namedActivityUri) is str:
            # it's a string so we need to validate it via REGEX
            # Django's URL validator
            regex = re.compile(
                    r'^(?:http|ftp)s?://' # http:// or https://
                    r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|' #domain...
                    r'localhost|' #localhost...
                    r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})' # ...or ip
                    r'(?::\d+)?' # optional port
                    r'(?:/?|[/?]\S+)$',
                    re.IGNORECASE)
            # our modified validator
            # TODO: implement different from Django's
            if regex.match(namedActivityUri):
                self.namedActivityUri = namedActivityUri
        else:
            raise TypeError('wasAssociatedWith must be an Activity, not a %s' % type(namedActivityUri))

    def make_graph(self):
        """
        Specialises ProvActivity.make_graph()

        :return: an rdflib Graph object
        """
        ProvActivity.make_graph(self)

        # add in a type of PROMS Activity
        PROMS = Namespace('http://promsns.org/def/proms#')
        self.g.add((URIRef(self.uri),
                    RDF.type,
                    PROMS.Entity))

        if self.namedActivityUri:
            self.g.add((URIRef(self.uri),
                        PROMS.namedActivityUri,
                        URIRef(self.wasInformedBy)))


from rdflib import URIRef, Namespace
from rdflib.namespace import RDF
from pyproms.proms_report import PromsReport
from pyproms.prov_activity import ProvActivity
from pyproms.proms_activity import PromsActivity
from proms_error import PromsDataModelError


class PromsInternalReport(PromsReport):
    """
    Creates a PROMS-O Internal Report instance
    
    This has the set of all Activities and an ending activity in addition to a Report class object
    """
    def __init__(self,
                 label,
                 reportingSystem,
                 nativeId,
                 startingActivity,
                 endingActivity,
                 all_activities,
                 comment=None):
        
        PromsReport.__init__(self,
                             label,
                             reportingSystem,
                             nativeId,
                             startingActivity,
                             comment)

        self.__set_endingActivity(startingActivity, endingActivity)
        self.__set_all_activities(all_activities)

    def __set_endingActivity(self, startingActivity, endingActivity):
        if (type(endingActivity) is ProvActivity or
            type(endingActivity) is PromsActivity):
            if startingActivity is not None:
                if endingActivity == startingActivity:
                    raise PromsDataModelError('For an Internal Report, the ending activity and starting activity must not be the same')
            self.endingActivity = endingActivity
        else:
            raise TypeError('endingActivity must be an Agent, not a %s' % type(endingActivity))

    def __set_all_activities(self, all_activities):
        if all(isinstance(n, ProvActivity) for n in all_activities):
            self.all_activities = all_activities
        else:
            raise TypeError('all_activities must be a list of Activity objects')

    def make_graph(self):
        """
        Specialises PromsReport.make_graph()

        :return: an rdflib Graph object
        """
        PromsReport.make_graph(self)

        PROMS = Namespace('http://promsns.org/def/proms#')

        self.g.add((URIRef(self.uri),
                    RDF.type,
                    PROMS.InternalReport))

        for activity in self.all_activities:
            # add the Activity to the graph
            self.g = self.g + activity.get_graph()

        # remove the triple that points the endingActivity to the startingActivity in the Report class
        self.g.remove((None, PROMS.endingActivity, None))
        # re-add the endingActivity triple pointing to the new Activity
        self.g.add((URIRef(self.uri),
                    PROMS.endingActivity,
                    URIRef(self.endingActivity.uri)))




from setuptools import setup
#from distutils.core import setup

setup(
    name='pyproms',
    packages=['pyproms'],
    version='1.1.7',
    author='Nicholas Car',
    author_email='nicholas.car@csiro.au',
    url='https://stash.csiro.au/projects/EIS/repos/proms-clients-python',
    download_url='https://stash.csiro.au/plugins/servlet/archive/projects/EIS/repos/proms-clients-python?at=refs%2Fheads%2Fmaster',
    license='LICENSE.txt',
    description='PyPROMS is a client library to make PROV and PROMS documents and submit them to a PROMS server instance',
    keywords=['rdf', 'prov', 'prov-o', 'proms', 'proms-o', 'provenance'],
    long_description='''
PyProms is a library of classes that helps Python coders generate
provenance reports using the PROV and PROMS Ontologies. It also has a
function to send a Report to an instances of the PROMS Server for
storage.

The classes match the PROMS-O OWL classes, for example the Python class
'Report' matches the PROMS-O 'Report' class.

PyProms is expected to be extended so if you need so add to it to get your
reporting job done, please do so! If you're really nice, you might consider
letting the author, Nicholas Car <nicholas.car@csiro.au> know what you've
added as this will help with further releases.
    ''',
    install_requires=['rdflib >= 3.0.0'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Topic :: Utilities',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)'
    ],
)

#use http://peterdowns.com/posts/first-time-with-pypi.html


from rdflib import URIRef, Namespace
from rdflib.namespace import RDF
from pyproms.proms_report import PromsReport


class PromsExternalReport(PromsReport):
    """
    Creates a PROMS-O External Report instance
    
    This has no new features on top of Report but it's worth maintaining the separate classes
    """
    def __init__(self,
                 label,
                 reportingSystem,
                 nativeId,
                 reportActivity,
                 comment=None):
        
        PromsReport.__init__(self,
                             label,
                             reportingSystem,
                             nativeId,
                             reportActivity,
                             comment)

    def make_graph(self):
        """
        Specialises PromsReport.make_graph()

        :return: an rdflib Graph object
        """
        PromsReport.make_graph(self)

        PROMS = Namespace('http://promsns.org/def/proms#')

        # The only thing we need to do here is to redefine the Report class as BasicReport.
        # There are no additional properties as the input & output Entities are part of the Activity
        self.g.add((URIRef(self.uri),
                    RDF.type,
                    PROMS.ExternalReport))


from rdflib import URIRef, Namespace, Literal
from rdflib.namespace import RDF
from pyproms.owlclass import OwlClass
from pyproms.proms_reportingsystem import PromsReportingSystem
from pyproms.prov_activity import ProvActivity
from pyproms.proms_activity import PromsActivity
import uuid


class PromsReport(OwlClass):
    """
    Creates a PROV-O Report instance
    """
    def __init__(self,
                 label,
                 reportingSystem,
                 nativeId,
                 reportActivity,
                 comment=None):

        OwlClass.__init__(self, label, comment)

        self.uri = 'http://placeholder.org#' + str(uuid.uuid4())
        self.__set_reportingSystem(reportingSystem)
        self.__set_nativeId(nativeId)
        self.__set_reportActivity(reportActivity)

    def __set_reportingSystem(self, reportingSystem):
        if type(reportingSystem) is PromsReportingSystem:
            self.reportingSystem = reportingSystem
        else:
            raise TypeError('reportingSystem must be a PromsReportingSystem, not a %s' % type(reportingSystem))

    def __set_nativeId(self, nativeId):
        if type(nativeId) is str:
            self.nativeId = nativeId
        else:
            raise TypeError('nativeId must be a String (str), not a %s' % type(nativeId))

    def __set_reportActivity(self, reportActivity):
        if (type(reportActivity) is ProvActivity or
            type(reportActivity) is PromsActivity):
            self.reportActivity = reportActivity
        else:
            raise TypeError('reportActivity must be an Agent, not a %s' % type(reportActivity))

    def make_graph(self):
        """
        Specialises RdfClass.make_graph()

        :return: an rdflib Graph object
        """
        OwlClass.make_graph(self)

        XSD = Namespace('http://www.w3.org/2001/XMLSchema#')
        PROMS = Namespace('http://promsns.org/def/proms#')

        self.g.add((URIRef(self.uri),
                    RDF.type,
                    PROMS.Report))

        self.g = self.g + self.reportingSystem.get_graph()
        self.g.add((URIRef(self.uri),
                    PROMS.reportingSystem,
                    URIRef(self.reportingSystem.uri)))

        self.g.add((URIRef(self.uri),
                    PROMS.nativeId,
                    Literal(self.nativeId, datatype=XSD.string)))

        self.g = self.g + self.reportActivity.get_graph()

        self.g.add((URIRef(self.uri),
                    PROMS.startingActivity,
                    URIRef(self.reportActivity.uri)))

        self.g.add((URIRef(self.uri),
                    PROMS.endingActivity,
                    URIRef(self.reportActivity.uri)))


from rdflib import URIRef, Literal, Namespace
from rdflib.namespace import RDF
import datetime
from pyproms.owlclass import OwlClass
from pyproms.prov_agent import ProvAgent


# TODO: allow Entity - Entity relationships
class ProvEntity(OwlClass):
    """
    Creates a PROV-O Entity instance
    """
    def __init__(self,
                 label,
                 uri=None,
                 comment=None,
                 wasAttributedTo=None,
                 creator=None,
                 created=None,
                 licence=None,
                 metadataUri=None,
                 downloadURL=None,
                 value=None):

        OwlClass.__init__(self, label, uri, comment)

        if wasAttributedTo:
            self.set_wasAttributedTo(wasAttributedTo)
        else:
            self.wasAttributedTo = None
        if creator:
            self.set_creator(creator)
        else:
            self.creator = None
        # default created time is now
        if created:
            self.created = created
        else:
            self.created = datetime.datetime.now()
        self.licence = licence
        self.metadataUri = metadataUri
        self.downloadURL = downloadURL
        if value:
            self.set_value(value)
        else:
            self.value = None

    def set_wasAttributedTo(self, wasAttributedTo):
        if type(wasAttributedTo) is ProvAgent:
            self.wasAttributedTo = wasAttributedTo
        else:
            raise TypeError('wasAttributedTo must be an Agent, not a %s' % type(wasAttributedTo))

    def set_creator(self, creator):
        if type(creator) is ProvAgent:
            self.creator = creator
        else:
            raise TypeError('creator must be an Agent, not a %s' % type(creator))

    def set_created(self, created):
        if type(created) is datetime.datetime:
            self.created = created
        else:
            raise TypeError('startedAtTime must be a datetime.datetime, not a %s' % type(created))

    def set_licence(self, licence):
        self.licence = licence

    def set_metadataUri(self, metadataUri):
        self.metadataUri = metadataUri

    def set_downloadURL(self, downloadURL):
        self.downloadURL = downloadURL

    def set_value(self, value):
        if (type(value) is str or
            type(value) is float or
            type(value) is int or
            type(value) is datetime.datetime):
            self.value = value
        else:
            raise TypeError('Entity \'value\' must be a string, int, float or datetime, not a %s' % type(value))

    def make_graph(self):
        """
        Specialises RdfClass.make_graph()

        :return: an rdflib Graph object
        """
        OwlClass.make_graph(self)

        PROV = Namespace('http://www.w3.org/ns/prov#')
        XSD = Namespace('http://www.w3.org/2001/XMLSchema#')
        PROMS = Namespace('http://promsns.org/def/proms#')
        DC = Namespace('http://purl.org/dc/elements/1.1/')
        DCAT = Namespace('http://www.w3.org/ns/dcat#')

        self.g.add((URIRef(self.uri),
                    RDF.type,
                    PROV.Entity))

        if self.wasAttributedTo:
            self.g = self.g + self.wasAttributedTo.get_graph()
            self.g.add((URIRef(self.uri),
                        PROV.wasAttributedTo,
                        URIRef(self.wasAttributedTo.uri)))

        if self.creator:
            self.g = self.g + self.creator.get_graph()
            self.g.add((URIRef(self.uri),
                        PROV.creator,
                        URIRef(self.creator.uri)))
        
        if self.created:
            self.g.add((URIRef(self.uri),
                        DC.created,
                        Literal(self.created.strftime("%Y-%m-%dT%H:%M:%S"), datatype=XSD.dateTime)))
        # TODO: review PROMS-O propertycreatedAtTimeDiffStep
        '''
        else:
            if isinstance(self.created, int):
               self.g.add((URIRef(self.uri),
                           PROMS.createdAtTimeDiffStep,
                           Literal(self.created, datatype=XSD.integer)))
        '''

        if self.licence:
            self.g.add((URIRef(self.uri),
                        DC.licence,
                        Literal(self.metadataUri, datatype=XSD.anyUri)))

        if self.metadataUri:
            self.g.add((URIRef(self.uri),
                        PROMS.metadataUri,
                        Literal(self.metadataUri, datatype=XSD.anyUri)))

        if self.downloadURL:
            self.g.add((URIRef(self.uri),
                        DCAT.downloadURL,
                        Literal(self.downloadURL, datatype=XSD.anyUri)))

        if self.value:
            if (type(self.value) is datetime.datetime):
                self.g.add((URIRef(self.uri),
                            PROV.value,
                            Literal(self.value, datatype=XSD.dateTime)))
            elif (type(self.value) is int):
                self.g.add((URIRef(self.uri),
                            PROV.value,
                            Literal(self.value, datatype=XSD.integer)))
            elif (type(self.value) is float):
                self.g.add((URIRef(self.uri),
                            PROV.value,
                            Literal(self.value, datatype=XSD.float)))
            else:  # string
                self.g.add((URIRef(self.uri),
                            PROV.value,
                            Literal(self.value, datatype=XSD.string)))

    def get_graph(self):
        """
        Generates the RDF graph of this class

        :return: This class's RDF graph
        """
        if not self.g:
            self.make_graph()

        return self.g